# SPDX-License-Identifier: GPL-2.0-or-later

"""
All of these effects will be DBus methods
"""

# pylint: disable=wildcard-import
from openrazer_daemon.dbus_services.dbus_methods.all import *
from openrazer_daemon.dbus_services.dbus_methods.chroma_keyboard import *
from openrazer_daemon.dbus_services.dbus_methods.bw2013 import *
from openrazer_daemon.dbus_services.dbus_methods.mamba import *
from openrazer_daemon.dbus_services.dbus_methods.macro import *
from openrazer_daemon.dbus_services.dbus_methods.keypad import *
from openrazer_daemon.dbus_services.dbus_methods.deathadder_chroma import *
from openrazer_daemon.dbus_services.dbus_methods.nagahexv2 import *
from openrazer_daemon.dbus_services.dbus_methods.nagahex import *
from openrazer_daemon.dbus_services.dbus_methods.lanceheadte import *
from openrazer_daemon.dbus_services.dbus_methods.kraken import *
from openrazer_daemon.dbus_services.dbus_methods.accessory import *
from openrazer_daemon.dbus_services.dbus_methods.charging_pad_chroma import *
from openrazer_daemon.dbus_services.dbus_methods.mouse_scroll_wheel import *
from openrazer_daemon.dbus_services.dbus_methods.argb_controller import *
